/*========================================================================================================
                                                                          
        ARTICLE: The Real Effects of FAS 166/167 on Banks Mortgage Approval and Sale Decisions                      
        AUTHORS: Yiwei Dou, Stephen G. Ryan, Biqin Xie                  
        JOURNAL OF ACCOUNTING RESEARCH 

 		The code below creates bank-year level variables using data from FR Y-9C reports.

 =========================================================================================================*/



/*==================================================
Create bank-year level variables using data from FR Y9C reports
/*==================================================*/

libname sec 'F:\bxie'; run;


/*Step 1: extract needed variables from annual FR Y9C reports during 2003Q4-2014Q4 downloaded from Fed Reserve Chicago website. The resulting dataset is named sec.k1*/
/*Step 2: created variables based on Y9C*/

/*merge hand collected consolidated VIE data for 2010 from sec.hand_collected_vie_2010 into sec.k1*/
proc sql; create table k2 
	as select a.*, b.vie_all_million 
	from sec.k1 as a 
	left join sec.hand_collected_vie_2010 as b 
	on a.rid=b.rid and a.yr=b.yr; quit; *103879; 


data k2; set k2;
	date=input(put(RSSD9999, Z8.), yymmdd8.); 
	month=month(date); 
	yr=year(date); 

/*	keep only annual Y9C reports*/
	if month=12; 
	if month=12 then do; qtr=4; end;
	q=(yr-1987)*4+qtr; 

/*	to avoid double counting, require RSSD9348=0 so as to keep only top BHCs instead of subsidiary of BHCs */
	if RSSD9348 ne 0 then delete; run;

data k3 (rename=(RSSD9001=rid RSSD9017=name BHCK2170=ta BHCK3210=teq BHCK4340=ni BHCK2948=tliab  
    BHCK2122=loan BHCK4107=ii BHCK4079=nii BHCK8274=t1cap BHCKA223=ta_riskweight 
	BHCK1754=htmb BHCK1773=afsm)); 
	set k2; 
run;  

/*get hand collected consolidated VIE data for 2010 from sec.hand_collected_vie_2010 */
proc sql; create table k4 
	as select a.*, b.vie_all_million 
	from k3 as a 
	left join sec.hand_collected_vie_2010 as b 
	on a.rid=b.rid and a.yr=b.yr; quit; 


data k4; set k4;
	/*	three types of on-B/S VIEs*/
	securi=sum(BHCKJ981,  BHCKJ984,  BHCKJ987,  BHCKJ990, BHCKJ993,  BHCKJ996,  BHCKK003,  BHCKK006,  BHCKK009,  BHCKK012, BHCKK030);

	abcp=sum(BHCKJ982, BHCKJ985,  BHCKJ988, BHCKJ991,  BHCKJ994, BHCKJ997,  BHCKK004, BHCKK007, BHCKK010,  BHCKK013, BHCKK031);

	othervie=sum( BHCKJ983, BHCKJ986,  BHCKJ989,  BHCKJ992, BHCKJ995, BHCKJ998,  BHCKK005, BHCKK008,  BHCKK011,  BHCKK014, BHCKK032);

	vie_all=sum(securi, abcp, othervie);
	label vie_all="total assets of consolidated VIEs";

	*define total off-B/S securitization;
	secuob=sum(BHCKB705, BHCKB706, BHCKB707, BHCKB708, BHCKB709, BHCKB710, BHCKB711);
	label secuob="outstanding balance of off-B/S securitized assets";

run; *103879. only annual data; 


/*create k1 for creating lead variables*/
proc sort data=k4 nodupkey; by rid descending yr; run;
data k4; set k4; 
	k1=.; l1=lag(rid); y1=lag(yr); if rid=l1 and yr=y1-1 then k1=1; run;

/*create lead of vie_all. if vie_all for 2011 is >0 but via_all_million for 2010 is missing (not able to hand collect),*/
/*then this probably means the bank is private and so should set its vie_all equal to missing for 2010, but should set the vie_all*/
/*of all other banks to be zero*/
proc sort data=k4; by rid descending yr; run;
data k4; set k4;
	lead_vie_all=lag(vie_all)*k1; run;

data k4; set k4;
	if yr eq 2010 then do; 
	if vie_all_million gt 0 then vie_all=vie_all_million*1000; 
	if vie_all_million=. and lead_vie_all gt 0  then vie_all=.; 
	if vie_all_million=. and lead_vie_all eq 0  then vie_all=0; end;

	
	if sum(ta, -vie_all) gt 0 then do;
	vieall_exvieall_r=vie_all/sum(ta, -vie_all);
	secuob_exvieall_r=secuob/sum(ta, -vie_all); end;

	label vieall_exvieall_r="Assets held by consolidated VIEs, scaled by adjusted TA";
	label secuob_exvieall_r="Off-balance sheet securitized assets, scaled by adjusted TA"; run;

proc sort data=k4 nodupkey; by rid yr; run; 
data k4; set k4; 
	c1=.; l1=lag(rid); y1=lag(yr); if rid=l1 and yr=y1+1 then c1=1; run;
proc sort data=k4 nodupkey; by rid yr; run; 
data s1424; set s1424;
	lta=lag(ta)*c1; 
	lt1rcapr=lag(t1rcapr)*c1; run;

/*take out positive values of secuob_exvieall_r, winsorize at 1% and 99% for the entire sample period, and add it back to main dataset*/
data p1; set k4;
	if secuob_exvieall_r gt 0; run;

%wins_bynone(dsetin=p1, dsetout=p2, byvar=none, vars=secuob_exvieall_r, type=winsor, pctl=1 99);

data k4; set k4;
	rename secuob_exvieall_r=secuob_r_win99positive_allyrs; run;

proc sql; create table k4 
	as select a.*, b.secuob_r_win99positive_allyrs 
	from k4 as a 
	left join p2 as b 
	on a.rid=b.rid 
	and a.yr=b.yr; quit;

data k4; set k4; 
	if secuob_r_win99positive_allyrs=. then secuob_r_win99positive_allyrs=0; 

	if ta gt 0 then sec_ta_r=sum(afsm,htmb)/ta;
	label sec_ta_r="(AFS+HTM)/TA ratio";

	if BHCK4059=. then BHCK4059=0; 
	if 3<=q<=14 then do; ii_loan=BHCK4393+BHCK4059+BHCK4065; end; 
	if 15<=q<=56 then do; ii_loan=BHCK4393+BHCK4059+(BHCK4503+BHCK4504)+(BHCK4505+BHCK4307); end; 
	if 2001<=yr<=2007 then do; ii_loan=BHCK4010+BHCK4059+BHCK4065; end;
	if yr>=2008 then do; ii_loan=BHCK4435+BHCK4436+BHCKF821+BHCK4059+BHCK4065; end;
	label ii_loan="Interest income from loans";

	absi=abs(ii)+abs(nii);
	if absi gt 0 then do; filoan=ii_loan/absi; end;
	label filoan="interest income from loans/total income";

	if q ge 15 then do; loanpdue2=sum(BHCK5526, BHCK5525); end; 
	label loanpdue2="total past due loans";

	if loan gt 0 then do; 
	ploanr2=loanpdue2/loan; end;
	label ploanr2="problem-loan to total-loan ratio";


	cd=sum(BHCB2210, BHCB3187, BHCB2389, BHCB6648, BHOD3189, BHOD3187, BHOD2389, BHOD6648);
	label cd="core deposit";
	if ta gt 0 then do; 
	cdr=cd/ta; end; 
	label cdr="core deposit scaled by average total assets";

	if lta gt 0 then do; 
	ch_wsf_rta=(wholesale_f-lwholesale_f)/lta; end;
	label ch_wsf_rta="annual growth in wholesale funding/lagged assets";

	a_deposit_w_int=(deposit_w_int+ldeposit_w_int)/2;
	a_money_w_int=(money_w_int+lmoney_w_int)/2;

	if a_deposit_w_int gt 0 then 
	deposit_rate=interest_dep/a_deposit_w_int;

	label deposit_rate="deposit rate";

	/*dependence on short term funding*/
	if ta gt 0 then do;
	st_funding=sum(bhck2309, bhck2332, bhckb995)/ta; end;

	if bhck4230~=. then provision=bhck4230;
	if bhck4230=. then provision=0;

	if loan gt 0 and lloan gt 0 then aloan=(loan+lloan)/2;

	if loan gt 0 and lloan=. then aloan=loan;

	if aloan>0 then provision_loan_r=provision/aloan; 

	if bhckc216~=. then Goodwill_impair=BHCKC216;
	if bhckc216=. then Goodwill_impair=0;

	if yr le 2008 then stress=0; if yr eq 2010 then stress=0;
	if yr eq 2009 and lta gt 100000000 then stress=1; 
	if yr eq 2009 and lta le 100000000 then stress=0; 
	if yr eq 2011 and lta gt 100000000 then stress=1; 
	if yr eq 2011 and lta le 100000000 then stress=0; 
	if yr ge 2012 and lta gt 50000000 then stress=1; 
	if yr ge 2012 and lta le 50000000 then stress=0; 

	if yr lt 2010 then post=0; 
	if yr ge 2010 then post=1; 
run;


/*to calculate excess loan loss provision, get average provision_loan_r during 1995-2004 or whatever years available during 1995-2004  */
/*require at least 3 nonmissing provision_loan_r per bank*/
data d1; set k4;
	if provision_loan_r~=.; run;
proc sort data=d1; by rid; run;
proc univariate data=d1 noprint; var provision_loan_r; by rid;
	output out=s0 n=num_nonmissing_provision_loan_r; run; *7240;
proc means data=s0 n mean std min p1 p5 p10 p25 median p75 p90 p95 p99 max; var num_nonmissing_provision_loan_r; run;

proc sql; create table d1 
	as select a.*, b.num_nonmissing_provision_loan_r 
	from d1 as a 
	left join s0 as b 
	on a.rid=b.rid; quit; 

data d1; set d1;
	if num_nonmissing_provision_loan_r ge 3; run; 

proc sort data=d1; by rid; run;
proc univariate data=d1 noprint; var provision_loan_r; by rid;
	output out=s1 mean=bank_mean_provision_loan_r; run; 

/*get bank_mean_provision_loan_r from s1 into k4*/
proc sql; create table k4 
	as select a.*, b.bank_mean_provision_loan_r 
	from k4 as a 
	left join s1 as b 
	on a.rid=b.rid; quit;

/*find out the median of bank_mean_provision_loan_r for banks with ta>$500m, and use the median for banks that enter the sample after 2004 and hence has missing bank_mean_provision_loan_r*/
data d4; set k4;
	if ta gt 50000; run;
proc sort data=d4 nodupkey out=d5; by rid; run;
proc means data=d5 n mean std min p1 p5 p10 p25 median p75 p90 p95 p99 max; var bank_mean_provision_loan_r;run;
/*note: median bank_mean_provision_loan_r is 0.0033023 for banks with ta>$500m*/

data k4; set k4;
	if bank_mean_provision_loan_r=. then bank_mean_provision_loan_r=0.0033023; run; 

/*merge in hand collected OTTI that is included in earnings during 2008 and 2009 from dataset sec.otti_20082013*/
proc sql; create table k4 as select 
	a.*, b.otti_ni 
	from k4 as a 
	left join sec.otti_20082013 as b 
	on a.rid=b.rid and 2008<=a.yr=b.yr<=2009; quit;

data k4; set k4;
	if yr ge 2010 then otti_ni=BHCKJ321; run;

data k4; set k4;
	if otti_ni=. then otti_ni=0; 

	normal_provision=bank_mean_provision_loan_r*aloan; 
	excess_provision2=provision-normal_provision; 

	crisis_loss2=sum(excess_provision2, goodwill_impair, otti_ni);
	label crisis_loss2="sum of excess provision2, goodwill impairment, and otti_ni";

	ata=(ta+lta)/2;

	if ata gt 0 then do;
	roa_crisis_loss2=crisis_loss2/ata;  
	roa_ex_crisis_loss2=sum(ni, crisis_loss2)/ata; end;

	lt1rcapr_d=lt1rcapr/100;
	t1rcapr_d=t1rcapr/100; run;

data k4; set k4;
	drop lt1rcapr t1rcapr; run;

data k4; set k4;
	rename lt1rcapr_d=lt1rcapr; 
	rename t1rcapr_d=t1rcapr; run;

/*replace lt1rcapr with t1rcapr if lt1rcapr is missing, so as to include in our sample the few banks in the first quarter when they become BHCs*/
data k4; set k4;
	if lt1rcapr~=. then lt1rcapr2=lt1rcapr;
	if lt1rcapr=. and t1rcapr~=. then lt1rcapr2=t1rcapr; run;

*convert TA to 2006 dollars before using annual cpi data from sec.cpi_annual;
proc sql; create table k4 as select a.*, b.cpi_annual_convertto_06 
	from k4 as a 
	left join sec.cpi_annual as b 
	on a.yr=b.yr; quit; 

data k4; set k4;
	ta06=ta*cpi_annual_convertto_06;
	if ta06 gt 0 then 
	size06=log(ta06); run;

proc sort data=k4; by rid yr; run;
data k4; set k4;
	lroa_crisis_loss2=lag(roa_crisis_loss2)*c1;
	lroa_ex_crisis_loss2=lag(roa_ex_crisis_loss2)*c1; 
	lsec_ta_r=lag(sec_ta_r)*c1;  
	lfiloan=lag(filoan)*c1; 
	lploanr2=lag(ploanr2)*c1;  
	lcdr=lag(cdr)*c1;  
	ldeposit_rate=lag(deposit_rate)*c1; 
	lsize06=lag(size06)*c1; run;

/*calculate the impact of VIE consolidation on tier 1 risk-based capital ratio*/
data k4; set k4;
	viecash=sum(bhckj981, bhckj982, bhckj983);
	viehtm=sum(bhckj984, bhckj985, bhckj986);
	vieafs=sum(bhckj987, bhckj988, bhckj989);
	vieresell=sum(bhckj990, bhckj991, bhckj992);
	vietrading=sum(bhckk003, bhckk004, bhckk005);
	viederiv=sum(bhckk006, bhckk007, bhckk008);
	vieloansale=sum(bhckj993, bhckj994, bhckj995);
	vieloan=sum(bhckj996, bhckj997, bhckj998);
	viere=sum(bhckk009, bhckk010, bhckk011);
	vieoth=sum(bhckk012, bhckk013, bhckk014);
	vieaoth=sum(bhckk030, bhckk031, bhckk032);
/*	group real estate owned, other, and all other VIEs assets together into vieothall */
	vieothall=sum(viere, vieoth,vieaoth);
	vieall=sum(bhckj999, bhckk001, bhckk002);


/*	calculate rwta of VIEs, may assume loans use 100% weight, while risk weights of all other assets are the same as in schedule HC-R of Y-9C for the bank*/
	rw_cash=sum(0.2*bhc20010, 0.5*bhc50010, 1*bhc90010)/bhck0010;
	rw_htm=sum(0.2*bhc21754, 0.5*bhc51754, 1*bhc91754)/bhcx1754;
	rw_afs=sum(0.2*bhc21773, 0.5*bhc51773, 1*bhc91773)/bhcx1773;
	rw_resell=sum(0.2*bhc2c225, 1*bhc9c225)/bhckc225;
	rw_trading=sum(0.2*bhc23545, 0.5*bhc53545, 1*bhc93545)/bhcx3545;
	rw_loansale=sum(0.2*bhc25369, 0.5*bhc55369, 1*bhc95369)/bhct5369;
	rw_loan=sum(0.2*bhc2b528, 0.5*bhc5b528, 1*bhc9b528)/bhctb528;
	rw_othall=sum(0.2*bhc2b639, 0.5*bhc5b639, 1*bhc9b639)/bhckb639;

	if rw_loansale=. and rw_loan>0 then rw_loansale=rw_loan; 
	if  rw_htm=. and rw_afs>0 then rw_htm=rw_afs;
	if rw_resell=. then rw_resell=0.20;
	if rw_trading=. then rw_trading=0.01; 

	rwta_vie=sum(rw_cash*viecash, rw_htm*viehtm, rw_afs*vieafs, rw_resell*vieresell, rw_trading*vietrading, 1*viederiv, rw_loansale*vieloansale, rw_loan*vieloan, rw_othall*vieothall, rw_afs*sum(viel, -viea));

	option=bhckj463;
	if option=. then option=0;

	rwta=bhcka223;
	rwta_asif=rwta-rwta_vie+option; 

	if rwta_asif gt 0 then do;
	t1rcapr_asif=sum(t1cap,vie_reserve)/rwta_asif; end;

	dift1rcapr=t1rcapr_asif-t1rcapr;

	if dift1rcapr lt 0 and dift1rcapr~=. then dift1rcapr=0; end; run;

data sec.data_y9c; set k4; run; *this dataset contains variables created using FR Y9C data;
